#!/usr/bin/tclsh
array set Orders {}
# загрузить список ордеров из файла CSV
proc LoadOrders { name } {
	global Orders
	set f [ open $name "r" ]
	if { $f == {} } {
		return -code error "Unable to open $name"
	}
	while { ! [ eof $f ] } {
		set str [ string trim [ gets $f ] ]
		if { $str == {} || [ string index $str 0 ] == "#" } {
			# пустые строки и комментарии пропускаем
			continue
		}
		set list [ split $str ";" ]
		set ticket [ string trim [ lindex $list 0 ] ]
		set Orders($ticket) $list
	}
	close $f
	return $Orders
}
# сохранить список ордеров в файл CSV
proc SaveOrders { name } {
	global Orders
	set f [ open $name "w" ]
	if { $f == {} } {
		return -code error "Unable to open $name"
	}
	foreach { ticket order } [ array get Orders ] {
		puts $f [ join $order ";" ]
	}
	close $f
	return [ array get Orders ]
}
# вернуть список ордеров
proc GetOrders {} {
	global Orders
	return [ array get Orders ]
}
# получить новый список ордеров 
# вернуть "разницу" - список новых, изменённых, удалённых
proc SetOrders { orderList } {
	global Orders
	set created {}
	set changed {}
	set removed {}
	foreach order $orderList {
		set ticket [ lindex $order 0 ]
		set known($ticket) $order
		if { ! [ info exists Orders($ticket) ] } {
			# новый тикет 
			# добавить в список свежесозданных
			lappend created $ticket
			set Orders($ticket) $order
			continue
		}
		if { $Orders($ticket) != $order } {
			# что-то поменялось 
			# добавить в список изменённых
			set Orders($ticket) $order
			lappend changed $ticket
		}
	}
	foreach ticket [ array names Orders ] {
		if { ! [ info exists known($ticket) ] } {
			# прежнего ордера нет в новом списке - значит удалён
			lappend removed $ticket
			unset Orders($ticket)
		}
	}
	return [ list $created $changed $removed ]
}
# should be 0


# по лиценз.бреду в Win исключён модуль Crypt.dll так что обойдёмся без подписей
# запретим подгрузку модуля Trf
package require uuid
package require tls

package forget Trf
package ifneeded Trf 2.0 {return -code error 0}

package require smtp
package require mime


oo::class create SMTP {

variable Servers 		; # list of servers. (default "localhost")
variable Ports	 		; # list of ports    (default 25)
variable UseTLS  		; # use TLS secure conn ? (default yes)
variable InsecureTLS	; # allow drop to insecure when handshake failed ? (default yes)
variable PortsTLS		; # ports for TLS conn. (default 2525)
variable UseAuth		; # use SMTP authorization ? (default no)
variable Username		; # username (default $USER)
variable Password		; # password (default "")
variable IdRealm		; # realm for Message-ID
	
constructor { {servers {localhost} } {ports { 25 } } } {
	set Servers $servers
	set Ports $ports
	set UseTLS yes
	set InsecureTLS yes
	set PortsTLS 2525
	set UseAuth no
	set User "user"
	set Password ""
	set IdRealm ""
} ; #/constructor 
method Auth { username password } {
	set UseAuth yes
	set Username $username 
	set Password $password
}
method NoAuth {} {
	set UseAuth no
}
method TLS { insecure {ports 2525} } {
	set UseTLS yes
	set InsecureTLS $insecure
	set PortsTLS $ports
}
method NoTLS {} {
	set UseTLS no
}
method SetIdRealm { realm } {
	set IdRealm $realm
}
method GenerateMessageID {} {
	return "<[ clock seconds ].[uuid::uuid generate]@atcl.luxtrade.tk>"
}
method SendMail { from to cc bcc subject body attach } {
	## собрать тело письма
	if { [ llength $attach ] == 0 } {
		# если нет аттачей то контент: plain/text
		set content [ mime::initialize -canonical "text/plain; charset=\"utf-8\"" -string $body ]
	} else {
		# если аттачи есть, то контент: plain/multipart
		# и состоит из нескольких частей
		set parts {}
		# первая часть - text/plain c текстом
		lappend parts [ mime::initialize -canonical "text/plain; charset=\"utf-8\"" -string $body ]
		foreach file $attach {
			# остальные части octet-stream
			set tail [ file tail $file ]
			set ext [ file extension $file ]
			switch -glob -nocase -- $ext {
				.png {
					lappend parts [ mime::initialize -canonical "image/png; name=\[$tail\]" \
									-file $file \
									-encoding base64 \
									-param [ list name [ file tail $file ] ] ] 
				}
				default {
					lappend parts [ mime::initialize -canonical "application/octet-stream; name=\[$tail\]" \
									-file $file \
									-encoding base64 \
									-param [ list name [ file tail $file ] ] ] 
				}
			}
		}
		# соберём весь multipart
		set content [ mime::initialize -canonical multipart/mixed -parts $parts ]
	}
	## Требуемые MIME заголовки
	mime::setheader $content Subject $subject
	mime::setheader $content Message-ID [ my GenerateMessageID ]
	## формировать команду отправки
	set command [ list smtp::sendmessage $content ]
	lappend command -header [ list From [ join $from "," ] ]
	lappend command -header [ list To 	[ join $to "," ] ]
	if { $cc != {} } {
		lappend command -header [ list Cc [ join $cc "," ] ]
	}
	if { $bcc != {} } {
		lappend command -header [ list Bcc	[ join $bcc "," ] ]
	}
	lappend command -servers $Servers
	## TLS	
	if { $UseTLS } {
		lappend command -usetls true -ports $PortsTLS 
		if { $InsecureTLS } {
			lappend command -tlspolicy [ list apply { args { return "insecure" } } ]
		} else {
			lappend command -tlspolicy [ list apply { args { return "secure" } } ]
		}
	} else {
		lappend command -ports $PortsTLS 
	}
	#lappend command -debug 1 
	## авторизация
	if { $UseAuth } {
		lappend command -username $Username -password $Password
	}
	set ret {}
	if { [ catch { {*}$command } msg ] } {
		set ret "Error:$msg : $from $to $cc $bcc $subject $body $attach "
	}
	mime::finalize $content
	if { $ret != {} } {
		return -code error $ret
	}
	
} ; # /method SendMail

	export Auth NoAuth
	export TLS NoTLS
	export SetIdRealm
	export SendMail
} ; # /class SMTP

##############################################
if { [ info exists argv0 ] && $argv0 == [ info script ] } {
	set SMTP [ SMTP new smtp.spaceweb.ru ]
	$SMTP Auth "luxtrade@nektomk.ru" ""
	#$SMTP TLS  yes
	# from to cc bcc subject attachs
	$SMTP SendMail "luxtrade@nektomk.ru" "luxtrade@nektomk.ru" {} {} "Subject" "Message body" {{screenshot.png}} 
}




